'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _mutationObserver = require('mutation-observer');

var _mutationObserver2 = _interopRequireDefault(_mutationObserver);

var _api = require('./api');

var _link = require('./link');

var _link2 = _interopRequireDefault(_link);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var noop = function noop() {};
var MUTATION_CONFIG = { attributes: true, childList: true, characterData: false, subtree: true };

var findActiveAnchor = function findActiveAnchor(position, anchors) {
  return anchors.find(function (_ref) {
    var parentId = _ref.parentId,
        _ref$configuration = _ref.configuration,
        offsetTop = _ref$configuration.offsetTop,
        offsetBottom = _ref$configuration.offsetBottom;

    var node = document.getElementById(parentId),
        _node$getBoundingClie = node.getBoundingClientRect(),
        top = _node$getBoundingClie.top,
        bottom = _node$getBoundingClie.bottom;


    return Math.round(top) <= position + offsetTop && Math.round(bottom) > position - offsetBottom;
  }) || null;
};

var getAnchorOwnProps = function getAnchorOwnProps(anchor) {
  var parentId = anchor.parentId,
      props = anchor.props;

  return { parentId: parentId, props: props };
};

var Navigation = function (_PureComponent) {
  _inherits(Navigation, _PureComponent);

  function Navigation() {
    _classCallCheck(this, Navigation);

    var _this = _possibleConstructorReturn(this, (Navigation.__proto__ || Object.getPrototypeOf(Navigation)).apply(this, arguments));

    _this.ticking = false;

    _this.state = {
      activeAnchor: null
    };

    _this.recalculate = _this.recalculate.bind(_this);
    _this.handleEvent = _this.handleEvent.bind(_this);
    _this.handleMutation = _this.handleMutation.bind(_this);
    _this.handleLinkClick = _this.handleLinkClick.bind(_this);
    _this.renderLink = _this.renderLink.bind(_this);

    _this.onScroll = _this.onScroll.bind(_this);
    _this.onEnter = _this.onEnter.bind(_this);
    return _this;
  }

  _createClass(Navigation, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      (0, _api.mount)(this);
      this.mutationObserver = new _mutationObserver2.default(this.handleMutation);
      this.mutationObserver.observe(document.body, MUTATION_CONFIG);

      window.addEventListener('scroll', this.recalculate, false);
      window.addEventListener('resize', this.recalculate, false);
      this.recalculate();

      if (this.props.anchors.length !== 0) {
        this.props.onChange(this.props.anchors.map(getAnchorOwnProps));
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps, prevState) {
      var prev = prevState.activeAnchor;
      var next = this.state.activeAnchor;

      if (prev && prev !== next) {
        this.onLeave(prev);
      }

      if (next && next !== prev) {
        this.onEnter(next);
      }

      if (this.props.anchors !== prevProps.anchors) {
        this.recalculate();
        this.props.onChange(this.props.anchors.map(getAnchorOwnProps));
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      (0, _api.unmount)();
      this.mutationObserver.disconnect();
      window.cancelAnimationFrame(this.raf);
      window.removeEventListener('scroll', this.recalculate);
      window.removeEventListener('resize', this.recalculate);
    }
  }, {
    key: 'onEnter',
    value: function onEnter(current) {
      var parentId = current.parentId,
          props = current.props;

      this.props.onEnter({ id: parentId, props: _extends({}, props) });
    }
  }, {
    key: 'onLeave',
    value: function onLeave(previous) {
      var parentId = previous.parentId,
          props = previous.props;

      this.props.onLeave({ id: parentId, props: _extends({}, props) });
    }
  }, {
    key: 'onScroll',
    value: function onScroll() {
      this.props.onScroll(window.scrollY || window.pageYOffset, this.state.activeAnchor);
    }
  }, {
    key: 'handleLinkClick',
    value: function handleLinkClick(id) {
      var anchor = this.props.anchors.find(function (a) {
        return a.parentId === id;
      });

      if (!anchor.configuration.disabled) {
        var element = document.getElementById(id);

        window.scroll({
          top: element.offsetTop - this.props.offset - anchor.configuration.offsetTop,
          behavior: this.props.behavior
        });
      }
    }
  }, {
    key: 'handleMutation',
    value: function handleMutation() {
      this.recalculate();
    }
  }, {
    key: 'handleEvent',
    value: function handleEvent() {
      var activeAnchor = findActiveAnchor(this.props.offset, this.props.anchors);

      this.setState({ activeAnchor: activeAnchor });

      this.ticking = false;
      this.mutationObserver.observe(document.body, MUTATION_CONFIG);
      this.onScroll();
    }
  }, {
    key: 'recalculate',
    value: function recalculate() {
      if (!this.ticking) {
        this.mutationObserver.disconnect();
        this.raf = window.requestAnimationFrame(this.handleEvent);
      }

      this.ticking = true;
    }
  }, {
    key: 'renderLink',
    value: function renderLink(anchor) {
      var _getAnchorOwnProps = getAnchorOwnProps(anchor),
          parentId = _getAnchorOwnProps.parentId,
          props = _getAnchorOwnProps.props;

      return _react2.default.createElement(_link2.default, {
        id: parentId,
        key: parentId,
        active: this.state.activeAnchor === anchor,
        onClick: this.handleLinkClick,
        childFactory: this.props.childFactory,
        childProps: props
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props,
          className = _props.className,
          anchors = _props.anchors,
          childFactory = _props.childFactory;


      return _react2.default.createElement(
        'span',
        { className: className },
        childFactory ? anchors.map(this.renderLink) : null
      );
    }
  }]);

  return Navigation;
}(_react.PureComponent);

Navigation.displayName = 'Navigation';
Navigation.propTypes = {
  childFactory: _propTypes2.default.func,
  anchors: _propTypes2.default.arrayOf(_propTypes2.default.shape({
    parentId: _propTypes2.default.string.isRequired,
    props: _propTypes2.default.shape({}),
    configuration: _propTypes2.default.shape({
      offsetTop: _propTypes2.default.number.isRequired,
      offsetBottom: _propTypes2.default.number.isRequired,
      disabled: _propTypes2.default.bool
    })
  })).isRequired,
  className: _propTypes2.default.string,
  offset: _propTypes2.default.number,
  behavior: _propTypes2.default.oneOf(['auto', 'smooth']),
  onScroll: _propTypes2.default.func,
  onEnter: _propTypes2.default.func,
  onLeave: _propTypes2.default.func,
  onChange: _propTypes2.default.func
};
Navigation.defaultProps = {
  offset: 0,
  behavior: 'smooth',
  onScroll: noop,
  onEnter: noop,
  onLeave: noop,
  onChange: noop
};
exports.default = Navigation;